@@ -127,6 +127,7 @@ enum {
 #define GDM_INTERRUPT_SELECT_USER 'U'
 #define GDM_INTERRUPT_LOGIN_SOUND 'L'
 #define GDM_INTERRUPT_THEME       'H'
+#define GDM_INTERRUPT_CUSTOM_CMD  'M'
 #define GDM_INTERRUPT_CANCEL      'X'
 
 /* List delimiter for config file lists */
@@ -184,7 +185,8 @@ enum {
  *
  * + The gui/gdmsetup.c program should be updated to support the new option
  *   unless there's a good reason not to (like it is a configuration value
- *   that only someone who really knows what they are doing should change).
+ *   that only someone who really knows what they are doing should change
+ *   like GDM_KEY_PID_FILE).
  * 
  * + Currently GDM treats any key in the "gui" and "greeter" categories,
  *   and security/PamStack as available for per-display configuration.  
@@ -226,13 +228,20 @@ enum {
 #define GDM_KEY_KILL_INIT_CLIENTS "daemon/KillInitClients=true"
 #define GDM_KEY_LOG_DIR "daemon/LogDir=" LOGDIR
 #define GDM_KEY_PATH "daemon/DefaultPath=" GDM_USER_PATH
-#define GDM_KEY_PID_FILE "daemon/PidFile=" GDM_PID_FILE
+#define GDM_KEY_PID_FILE "daemon/PidFile=/var/run/gdm.pid"
 #define GDM_KEY_POSTSESSION "daemon/PostSessionScriptDir=" GDMCONFDIR "/PostSession/"
 #define GDM_KEY_PRESESSION "daemon/PreSessionScriptDir=" GDMCONFDIR "/PreSession/"
 #define GDM_KEY_POSTLOGIN "daemon/PostLoginScriptDir=" GDMCONFDIR "/PreSession/"
 #define GDM_KEY_FAILSAFE_XSERVER "daemon/FailsafeXServer="
 #define GDM_KEY_X_KEEPS_CRASHING "daemon/XKeepsCrashing=" GDMCONFDIR "/XKeepsCrashing"
 #define GDM_KEY_REBOOT  "daemon/RebootCommand=" REBOOT_COMMAND
+#define GDM_KEY_CUSTOM_CMD_TEMPLATE "customcommand/CustomCommand"
+#define GDM_KEY_CUSTOM_CMD_LABEL_TEMPLATE "customcommand/CustomCommandLabel"
+#define GDM_KEY_CUSTOM_CMD_LR_LABEL_TEMPLATE "customcommand/CustomCommandLRLabel"
+#define GDM_KEY_CUSTOM_CMD_TEXT_TEMPLATE "customcommand/CustomCommandText"
+#define GDM_KEY_CUSTOM_CMD_TOOLTIP_TEMPLATE "customcommand/CustomCommandTooltip"
+#define GDM_KEY_CUSTOM_CMD_NO_RESTART_TEMPLATE "customcommand/CustomCommandNoRestart"
+#define GDM_KEY_CUSTOM_CMD_IS_PERSISTENT_TEMPLATE "customcommand/CustomCommandIsPersistent"
 #define GDM_KEY_ROOT_PATH "daemon/RootPath=/sbin:/usr/sbin:" GDM_USER_PATH
 #define GDM_KEY_SERV_AUTHDIR "daemon/ServAuthDir=" AUTHDIR
 #define GDM_KEY_SESSION_DESKTOP_DIR "daemon/SessionDesktopDir=/etc/X11/sessions/:" DMCONFDIR "/Sessions/:" DATADIR "/gdm/BuiltInSessions/:" DATADIR "/xsessions/"
@@ -292,6 +301,7 @@ enum {
 #define GDM_KEY_USER_MAX_FILE "security/UserMaxFile=65536"
 #define GDM_KEY_RELAX_PERM "security/RelaxPermissions=0"
 #define GDM_KEY_CHECK_DIR_OWNER "security/CheckDirOwner=true"
+#define GDM_KEY_SUPPORT_AUTOMOUNT "security/SupportAutomount=false"
 #define GDM_KEY_RETRY_DELAY "security/RetryDelay=1"
 #define GDM_KEY_DISALLOW_TCP "security/DisallowTCP=true"
 #define GDM_KEY_PAM_STACK "security/PamStack=gdm"
@@ -432,11 +442,31 @@ enum {
 typedef struct _GdmConnection GdmConnection;
 #endif  /* TYPEDEF_GDM_CONNECTION */
 
+#ifndef TYPEDEF_GDM_CUSTOM_CMD
+#define TYPEDEF_GDM_CUSTOM_CMD
+typedef struct _GdmCustomCmd GdmCustomCmd;
+#endif /* TYPEDEF_GDM_CUSTOM_CMD */
+struct _GdmCustomCmd {
+    gchar *command; /* command(s) to execute */
+    gchar *command_label; /* button/menu item label */
+    gchar *command_lr_label; /* radio button/list item label */
+    gchar *command_text; /* warning dialog text */
+    gchar *command_tooltip; /* tooltip string */
+    gboolean command_no_restart; /* no restart flag */
+    gboolean command_is_persistent; /* persistence flag */
+};
+
+#define GDM_CUSTOM_COMMAND_MAX 10 /* maximum number of supported custom commands */
+
+/* Values between GDM_LOGOUT_ACTION_CUSTOM_CMD_FIRST and 
+   GDM_LOGOUT_ACTION_CUSTOM_CMD_LAST are reserved and should not be used */
 typedef enum {
 	GDM_LOGOUT_ACTION_NONE = 0,
 	GDM_LOGOUT_ACTION_HALT,
 	GDM_LOGOUT_ACTION_REBOOT,
 	GDM_LOGOUT_ACTION_SUSPEND,
+	GDM_LOGOUT_ACTION_CUSTOM_CMD_FIRST,
+	GDM_LOGOUT_ACTION_CUSTOM_CMD_LAST = GDM_LOGOUT_ACTION_CUSTOM_CMD_FIRST + GDM_CUSTOM_COMMAND_MAX - 1,
 	GDM_LOGOUT_ACTION_LAST
 } GdmLogoutAction;
 
@@ -527,6 +557,8 @@ struct _GdmDisplay {
     gboolean logged_in; /* TRUE if someone is logged in */
     char *login;
 
+    char *preset_user;
+
     gboolean timed_login_ok;
 
     int screenx;
@@ -706,9 +738,17 @@ void		gdm_final_cleanup	(void);
 #define GDM_SOP_SUSPEND_MACHINE "SUSPEND_MACHINE"  /* no arguments */
 #define GDM_SOP_CHOSEN_THEME "CHOSEN_THEME"  /* <slave pid> <theme name> */
 
+/*Execute custom cmd*/
+#define GDM_SOP_CUSTOM_CMD "CUSTOM_CMD"  /* <slave pid> <cmd id> */
+
 /* Start a new standard X flexible server */
 #define GDM_SOP_FLEXI_XSERVER "FLEXI_XSERVER" /* no arguments */
 
+#define GDM_SOP_SHOW_ERROR_DIALOG "SHOW_ERROR_DIALOG"  /* show the error dialog from daemon */
+#define GDM_SOP_SHOW_YESNO_DIALOG "SHOW_YESNO_DIALOG"  /* show the yesno dialog from daemon */
+#define GDM_SOP_SHOW_QUESTION_DIALOG "SHOW_QUESTION_DIALOG"  /* show the question dialog from daemon */
+#define GDM_SOP_SHOW_ASKBUTTONS_DIALOG "SHOW_ASKBUTTON_DIALOG"  /* show the askbutton dialog from daemon */
+
 /* Notification protocol */
 /* keys */
 #define GDM_NOTIFY_ALLOW_REMOTE_ROOT "AllowRemoteRoot" /* <true/false as int> */
@@ -729,6 +769,7 @@ void		gdm_final_cleanup	(void);
 #define GDM_NOTIFY_SOUND_ON_LOGIN_FAILURE_FILE "SoundOnLoginFailureFile" /* <sound file> */
 #define GDM_NOTIFY_ADD_GTK_MODULES "AddGtkModules" /* <true/false as int> */
 #define GDM_NOTIFY_GTK_MODULES_LIST "GtkModulesList" /* <modules list> */
+#define GDM_NOTIFY_CUSTOM_CMD_TEMPLATE "CustomCommand" /* <custom command path> */
 
 /* commands, seel GDM_SLAVE_NOTIFY_COMMAND */
 #define GDM_NOTIFY_DIRTY_SERVERS "DIRTY_SERVERS"
@@ -743,6 +784,16 @@ void		gdm_final_cleanup	(void);
 #define GDM_SLAVE_NOTIFY_KEY '!'
 /* notify a command */
 #define GDM_SLAVE_NOTIFY_COMMAND '#'
+/* send the response */
+#define GDM_SLAVE_NOTIFY_RESPONSE 'R'
+/* send the error dialog response */
+#define GDM_SLAVE_NOTIFY_ERROR_RESPONSE 'E'
+/* send the yesno dialog response */
+#define GDM_SLAVE_NOTIFY_YESNO_RESPONSE 'Y'
+/* send the askbuttons dialog response */
+#define GDM_SLAVE_NOTIFY_ASKBUTTONS_RESPONSE 'B'
+/* send the question dialog response */
+#define GDM_SLAVE_NOTIFY_QUESTION_RESPONSE 'Q'
 
 /*
  * Maximum number of messages allowed over the sockets protocol.  This
@@ -821,6 +872,26 @@ void		gdm_final_cleanup	(void);
  *      200 = Too many messages
  *      999 = Unknown error
  */
+#define GDM_SUP_FLEXI_XSERVER_USER  "FLEXI_XSERVER_USER" /* <username> <xserver type> */
+/* FLEXI_XSERVER_USER: Start a new X flexible display and initialize the 
+ *                greeter with the given username. Only supported on 
+ *                connection that passed AUTH_LOCAL
+ * Supported since: 2.17.7
+ * Arguments: <username> <xserver type>
+ *   If no server type specified, starts the standard X server
+ * Answers:
+ *   OK <display>
+ *   ERROR <err number> <english error description>
+ *      0 = Not implemented
+ *      1 = No more flexi servers
+ *      2 = Startup errors
+ *      3 = X failed
+ *      4 = X too busy
+ *      6 = No server binary
+ *      100 = Not authenticated
+ *      200 = Too many messages
+ *      999 = Unknown error
+ */
 #define GDM_SUP_FLEXI_XNEST  "FLEXI_XNEST" /* <display> <uid> <xauth cookie> <xauth file> */
 /* FLEXI_XNEXT: Start a new flexible Xnest display.
  * Note:        Supported on older versions from 2.2.4.0, later
@@ -856,10 +927,35 @@ void		gdm_final_cleanup	(void);
  *      200 = Too many messages
  *      999 = Unknown error
  */
+#define GDM_SUP_FLEXI_XNEST_USER  "FLEXI_XNEST_USER" /* <username> <display> <uid> <xauth cookie> <xauth file> */
+/* FLEXI_XNEXT_USER: Start a new flexible Xnest display and 
+ *              initialize the greeter with the given username
+ * Note:        The cookie should be the MIT-MAGIC-COOKIE-1,
+ *              the first one gdm can find in the XAUTHORITY
+ *              file for this display.  If that's not what you
+ *              use you should generate one first.  The cookie
+ *              should be in hex form.
+ * Supported since: 2.17.7
+ * Arguments: <username> <display to run on> <uid of requesting user>
+ *            <xauth cookie for the display> <xauth file>
+ * Answers:
+ *   OK <display>
+ *   ERROR <err number> <english error description>
+ *      0 = Not implemented
+ *      1 = No more flexi servers
+ *      2 = Startup errors
+ *      3 = X failed
+ *      4 = X too busy
+ *      5 = Xnest can't connect
+ *      6 = No server binary
+ *      100 = Not authenticated
+ *      200 = Too many messages
+ *      999 = Unknown error
+ */
 #define GDM_SUP_ADD_DYNAMIC_DISPLAY	"ADD_DYNAMIC_DISPLAY" 
 /*
  * ADD_DYNAMIC_DISPLAY: Create a new server definition that will
- *                      run on the specified display leaving, it
+ *                      run on the specified display leaving it
  *                      in DISPLAY_CONFIG state.
  * Supported since: 2.8.0.0
  * Arguments: <display to run on>=<server>
@@ -931,7 +1027,7 @@ void		gdm_final_cleanup	(void);
  *   and instead of vt, it lists the parent display in standard form.
  *
  *   ERROR <err number> <english error description>
- *      1 = Not implemented
+ *      0 = Not implemented
  *      200 = Too many messages
  *      999 = Unknown error
  */
@@ -965,7 +1061,6 @@ void		gdm_final_cleanup	(void);
  *   ERROR <err number> <english error description>
  *      0 = Not implemented
  *      1 = No servers found
- *      50 = Unsupported key
  *      200 = Too many messages
  *      999 = Unknown error
  */
@@ -1135,8 +1230,8 @@ void		gdm_final_cleanup	(void);
  * Supported since: 2.5.90.0
  * Answers:
  *   OK <action>;<action>;...
- *      Where action is one of HALT, REBOOT or SUSPEND.  An empty list
- *      can also be returned if no action is possible.  A '!' is appended
+ *      Where action is one of HALT, REBOOT, SUSPEND or CUSTOM_CMDX (where X in [0,GDM_CUSTOM_COMMAND_MAX)).
+ *      An empty list can also be returned if no action is possible.  A '!' is appended
  *      to an action if it was already set with SET_LOGOUT_ACTION or
  *      SET_SAFE_LOGOUT_ACTION.  Note that SET_LOGOUT_ACTION has precedence
  *      over SET_SAFE_LOGOUT_ACTION.
@@ -1146,6 +1241,38 @@ void		gdm_final_cleanup	(void);
  *      200 = Too many messages
  *      999 = Unknown error
  */
+#define GDM_SUP_QUERY_CUSTOM_CMD_LABELS "QUERY_CUSTOM_CMD_LABELS" /* None */
+/* QUERY_CUSTOM_CMD_LABELS: Query labels belonging to exported custom commands
+ *                          Only supported on connections that passed
+ *                          AUTH_LOCAL.
+ * Supported since: 2.5.90.0
+ * Answers:
+ *   OK <label1>;<label2>;...
+ *      Where labelX is one of the labels belonging to CUSTOM_CMDX (where X in [0,GDM_CUSTOM_COMMAND_MAX)).
+ *      An empty list can also be returned if none of the custom commands are 
+ *      exported outside login manager (no CustomCommandIsPersistent options are set to true).  
+ *   ERROR <err number> <english error description>
+ *      0 = Not implemented
+ *      100 = Not authenticated
+ *      200 = Too many messages
+ *      999 = Unknown error
+ */
+#define GDM_SUP_QUERY_CUSTOM_CMD_NO_RESTART_STATUS "QUERY_CUSTOM_CMD_NO_RESTART_STATUS" /* None */
+/* QUERY_CUSTOM_CMD_NO_RESTART_STATUS: Query NoRestart config options for each of custom commands
+ *                                     Only supported on connections that passed
+ *                                     AUTH_LOCAL.
+ * Supported since: 2.5.90.0
+ * Answers:
+ *   OK <status>
+ *      Where each bit of the status represents NoRestart value for each of the custom commands.
+ *      bit on (1):  NoRestart = true, 
+ *      bit off (0): NoRestart = false.
+ *   ERROR <err number> <english error description>
+ *      0 = Not implemented
+ *      100 = Not authenticated
+ *      200 = Too many messages
+ *      999 = Unknown error
+ */
 #define GDM_SUP_SET_LOGOUT_ACTION "SET_LOGOUT_ACTION" /* <action> */
 /* SET_LOGOUT_ACTION: Tell the daemon to halt/restart/suspend after
  *                    slave process exits.  Only supported on
@@ -1156,6 +1283,7 @@ void		gdm_final_cleanup	(void);
  *   HALT           Set exit action to 'halt'
  *   REBOOT         Set exit action to 'reboot'
  *   SUSPEND        Set exit action to 'suspend'
+ *   CUSTOM_CMDX    Set exit action to 'custom command X' where where X in [0,GDM_CUSTOM_COMMAND_MAX)
  * Answers:
  *   OK
  *   ERROR <err number> <english error description>
@@ -1182,6 +1310,7 @@ void		gdm_final_cleanup	(void);
  *   HALT           Set exit action to 'halt'
  *   REBOOT         Set exit action to 'reboot'
  *   SUSPEND        Set exit action to 'suspend'
+ *   CUSTOM_CMDX    Set exit action to 'custom command X' where where X in [0,GDM_CUSTOM_COMMAND_MAX)
  *
  * Answers:
  *   OK
@@ -1192,10 +1321,11 @@ void		gdm_final_cleanup	(void);
  *      200 = Too many messages
  *      999 = Unknown error
  */
-#define GDM_SUP_LOGOUT_ACTION_NONE	"NONE"
-#define GDM_SUP_LOGOUT_ACTION_HALT	"HALT"
-#define GDM_SUP_LOGOUT_ACTION_REBOOT	"REBOOT"
-#define GDM_SUP_LOGOUT_ACTION_SUSPEND	"SUSPEND"
+#define GDM_SUP_LOGOUT_ACTION_NONE	          "NONE"
+#define GDM_SUP_LOGOUT_ACTION_HALT	          "HALT"
+#define GDM_SUP_LOGOUT_ACTION_REBOOT	          "REBOOT"
+#define GDM_SUP_LOGOUT_ACTION_SUSPEND	          "SUSPEND"
+#define GDM_SUP_LOGOUT_ACTION_CUSTOM_CMD_TEMPLATE "CUSTOM_CMD"
 /*
  */
 #define GDM_SUP_QUERY_VT "QUERY_VT" /* None */
